#######
# aggregateSnapshots.py
# description : group the archived and unarchived pages by url and aggregate their timestamped snapshots
# licence : AGPL + CECILL v3
# author : quentin lobbé - qlobbe@iscpif.fr
#######

# python3 aggregateSnapshots.py output/archives-firsttuesday.com-1996-2010.csv output/unarchived-1996-2010.csv output/siteTree-1996-2010.json

import sys
import csv
import json

pathArchives = ""
pathUnArchives = ""
pathTree = ""

try :
    pathArchives = sys.argv[1]
    pathUnArchives = sys.argv[2]
    pathTree = sys.argv[3]
except :
    print ("! args error\n")
    sys.exit(0)

inf = pathArchives.split('-')[2]
sup = pathArchives.split('-')[3].replace(".csv","")

def readJson(path) :
    file = open(path)
    return json.load(file)

urls = set()
tree = readJson(pathTree)
root = next(iter(tree))

def normalizedUrl(url) :
    url = url.replace("http://","").replace("www.","").replace(":80","")
    if url.endswith("/") :
        url = url[:-1]
    if (url.endswith(".htm")) :
        url = url.replace(".htm",".html")
    if url.startswith('/') :
        url = root + url
    if url.startswith('./') :
        url = root + url[1:]  
    if url.startswith('../') :
        url = root + url[2:]                
    return url.lower()

def indexUrls(path) :

	with open(path, newline='\n') as file :

	    rawArchives = csv.reader(file, delimiter=';')
	    next(rawArchives)

	    for row in rawArchives :

	        url = normalizedUrl(row[2])

	        urls.add(url)

# def indexSnapshot(row,isArchived) :

# 	# snapshot's status : 0 <=> unarchived;  1 <=> changed ; 2 <=> unchanged

# 	ts  = row[0]
# 	url = normalizedUrl(row[2])

# 	digest = row[3]

# 	if isArchived :

# 		if pages[url]['last_digest'] == digest :

# 			pages[url]['snapshots'].append({'timestamp':ts, 'status':2})

# 		else :

# 			pages[url]['snapshots'].append({'timestamp':ts, 'status':1})
# 			pages[url]['last_digest'] = digest

# 	else :

# 		pages[url]['snapshots'].append({'timestamp':ts, 'status':0})


def indexSnapshotBis(row) :

	ts  = row[0]
	url = normalizedUrl(row[2])
	digest = row[3]

	
def getFst(snapshot) :
	return int(snapshot[0])	


def aggregate(path,isArchived) :

	with open(path, newline='\n') as file :

		rawArchives = csv.reader(file, delimiter=';')
		next(rawArchives)

		if isArchived :

			# pathArchives

			for row in rawArchives :

				ts  = row[0]
				url = normalizedUrl(row[2])
				digest = row[3]	

				pages[url].append([ts,digest])

			for url in pages :

				pages[url].sort(key=getFst)

				if (len(pages[url]) > 0) :

					cur = ""
					acc = []

					for snapshot in pages[url] :

						if cur == snapshot[1] :
							acc.append({'timestamp':snapshot[0], 'status':2})
						else :
							acc.append({'timestamp':snapshot[0], 'status':1})
							cur = snapshot[1]

					pages[url] = {'snapshots':acc}

		else :

			# pathUnArchives

			for row in rawArchives :

				ts  = row[0]
				url = normalizedUrl(row[2])

				pages[url] = {'snapshots': [{'timestamp':ts, 'status':0}]}



def toMoments() :

	id = 0
	url_id = 0

	for url in pages :

		moments = []

		fst = pages[url]['snapshots'][0]['timestamp']
		lst = pages[url]['snapshots'][0]['timestamp']

		if (pages[url]['snapshots'][0]['status'] == 0) :
			
			# un-archived urls
			
			moments.append((id,(fst,lst)))
			id += 1
			pages[url] = {'status':0 , 'moments':moments, 'id':url_id}
			url_id += 1
		
		else :
		
			# archived urls

			for snapshot in pages[url]['snapshots'] :

				ts  = snapshot['timestamp']
				status = snapshot['status']

				if status == 2 :

					# un-changed 
					
					lst = ts

				elif ts != lst :

					# changed

					moments.append((id,(fst,lst)))
					id += 1
					fst = ts
					lst = ts

			moments.append((id,(fst,lst)))
			id += 1

			pages[url] = {'status':1 , 'moments':moments, 'id':url_id}
			url_id += 1

# index the urls

indexUrls(pathArchives)
indexUrls(pathUnArchives)

# init the aggregator

pages = dict()


for url in urls :
	pages[url] = []

# aggregate

aggregate(pathArchives,True)
aggregate(pathUnArchives,False)

# moments

toMoments()

# export

with open("output/moments-" + inf + "-" + sup + ".json", 'w') as file :
   
   json.dump(pages, file, sort_keys=True, indent=4)