#######
# getArchives.py
# description : get all the unique archived urls sharing the same prefix from the Internet Archives API
# licence : AGPL + CECILL v3
# author : quentin lobbé - qlobbe@iscpif.fr
#######

# python3 getArchives.py firsttuesday.com 500

import sys
import urllib.request
from bs4 import BeautifulSoup

prefix = ""
batchSize = ""

try :
    prefix = sys.argv[1]
    batchSize = sys.argv[2]
except :
    print ("! args error\n")
    sys.exit(0)


# harvesting years
years = ["1996","1997","1998","1999","2000","2001","2002","2003","2004","2005","2006","2007","2008","2009","2010"]

# cdx documentation available at https://github.com/internetarchive/wayback/blob/master/wayback-cdx-server/README.md
archiveReq = "http://web.archive.org/cdx/search/cdx?url=" + prefix + "&matchType=prefix&collapse=digest&filter=mimetype:text/html&showResumeKey=true&limit=" + batchSize

csv    = open("output/archives-" + prefix + "-" + years[0] + "-" + years[len(years) - 1] + ".csv", "w")
header = "timestamp;date;url;digest\n"
csv.write(header)

def getUrls(url) :
    try :
        req = urllib.request.Request(url)
        with urllib.request.urlopen(req) as res:
            return res.read()
    except Exception as e :
        print(str(e))
        return None


for year in years :
    print (year + " harvesting ...")
    next = True
    currentReq = archiveReq + "&from=" + year + "&to=" + year
    resumeKey = None
    while next :
        if resumeKey is None :
            urls = getUrls(currentReq)
        else :
            urls = getUrls(currentReq + "&resumeKey=" + resumeKey)

        if urls is None or urls == "" :
            next = False
        else :
            soup = BeautifulSoup(urls, 'html.parser')
            lines = soup.getText().split("\n")
            if len(lines) == 1 :
                resumeKey = None
                next = False
            else :
                if "html" in lines[len(lines) - 2] :
                    resumeKey = None
                    next = False
                else :
                    resumeKey = lines[len(lines) - 2]
                    lines = lines[:-2]

                for l in lines:
                    f = l.split(" ")
                    if len(f) < 6:
                        continue
                    timestamp = f[1]
                    y = timestamp[0:4]
                    m = timestamp[4:6]
                    d = timestamp[6:8]
                    date = d + "-" + m + "-" + y 
                    url = f[2]
                    digest = f[5]
                    row = timestamp + ";" + date + ";" + url + ";" + digest + "\n";
                    csv.write(row)                  