#######
# graphToCernes.py
# description : sort the archived an unarchived ids regarding their continuity links and href citations then saved a file of cernes
# licence : AGPL + CECILL v3
# author : quentin lobbé - qlobbe@iscpif.fr
#######

# python3 graphToCernes.py output/continuity-graph-urls-1996-2010.csv output/moments-1996-2010.json output/cliques-1996-2010.json output/siteTree-1996-2010.json output/continuity-graph-1996-2010.csv

import sys
import csv
import json
import networkx as nx
from community import community_louvain

pathContinuity = ""
pathMoments = ""
pathCliques = ""
pathTree = ""
pathContinuityMoments = ""

try :
    pathContinuity = sys.argv[1]
    pathMoments = sys.argv[2]
    pathCliques = sys.argv[3]
    pathTree = sys.argv[4]
    pathContinuityMoments = sys.argv[5]
except :
    print ("! args error\n")
    sys.exit(0)

inf = pathContinuity.split('-')[3]
sup = pathContinuity.split('-')[4].replace(".csv","")

def readJson(path) :
	file = open(path)
	return json.load(file)

urls_moments = readJson(pathMoments)
cliques = readJson(pathCliques)
tree = readJson(pathTree)
root = next(iter(tree))
ids = {}
edges = []
edgesMoments = []
cernes = {'from':inf, 'to':sup, 'root':root, 'urls':{}}

# compute some counting 

cpt_archived = 0
cpt_unarchived = 0
cpt_captures = 0

for url in urls_moments :
	if urls_moments[url]['status'] > 0 :
		cpt_archived += 1
		for moment in urls_moments[url]['moments'] :
			if moment[1][0] != moment[1][1] :
				cpt_captures += 2
			else :
				cpt_captures += 1 
	else : 
		cpt_unarchived += 1

cernes['nb_archived'] = cpt_archived
cernes['nb_unarchived'] = cpt_unarchived
cernes['nb_captures'] = cpt_captures

# re-dindex the continuity links

with open(pathContinuity, newline='\n') as file :

    rawEdges = csv.reader(file, delimiter=';')
    next(rawEdges)

    for row in rawEdges :

        edges.append((row[0],row[1]))
        ids[row[0]] = row[2]
        ids[row[1]] = row[3]

with open(pathContinuityMoments, newline='\n') as file :

    rawEdges = csv.reader(file, delimiter=';')
    next(rawEdges)

    for row in rawEdges :

        edgesMoments.append(str(row[0]) + '-' + str(row[1]))     

# compute the louvain communities

def getLstMoment(url_id) :
	url = ids[url_id]
	lst = urls_moments[url]['moments'][len(urls_moments[url]['moments']) - 1][1][1]
	return int(lst)

def getFstMoment(url_id) :
	url = ids[url_id]
	lst = urls_moments[url]['moments'][0][1][0]
	return int(lst)

def findFstInCommunity(community) :
	fst = getFstMoment(community[0])
	for url_id in community :
		if getFstMoment(url_id) < fst :
			fst = getFstMoment(url_id)
	return fst

def findLstInCommunity(community) :
	lst = getLstMoment(community[0])
	for url_id in community :
		if getLstMoment(url_id) > lst :
			lst = getLstMoment(url_id)
	return lst

def getClosestInf(thr,communities) :
	res = communities[0]
	dif = abs(thr - findFstInCommunity(res))
	for com in communities :
		cur = abs(thr - findFstInCommunity(com))
		if cur < dif :
			dif = cur
			res = com
	return res

def getClosestSup(thr,communities) :
	res = communities[0]
	dif = abs(thr - findLstInCommunity(res))
	for com in communities :
		cur = abs(thr - findLstInCommunity(com))
		if cur < dif :
			dif = cur
			res = com
	return res

def sortUrlsInPartitions(urls) :

	byCommunity = []

	cur = (list(urls.values()))[0]
	acc = []

	for url in urls :
		if urls[url] == cur :
			acc.append(url)
		else :
			cur = urls[url]
			byCommunity.append(acc)
			acc = [url] 

	acc = []

	for com in byCommunity :
		if len(com) < 10 :
			acc = acc + com

	byCommunityAggregated = list(filter(lambda com: (len(com) > 11), byCommunity)) + [acc]

	byCommunityAggregatedSorted = []
	thr = 0

	for idx in range(len(byCommunityAggregated)) :
		if len(byCommunityAggregated) < 1 :
			continue

		if (idx + 1) % 2 != 0 :
			candidate = getClosestInf(thr,byCommunityAggregated)
			byCommunityAggregated.remove(candidate)
			candidate.sort(key=getLstMoment)
			thr = findLstInCommunity(candidate)
			byCommunityAggregatedSorted.append(candidate)
		else :
			candidate = getClosestSup(thr,byCommunityAggregated)
			byCommunityAggregated.remove(candidate)
			candidate.sort(key=getLstMoment,reverse=True)
			thr = findFstInCommunity(candidate)
			byCommunityAggregatedSorted.append(candidate)

	byCommunityFlatten = [item for sublist in byCommunityAggregatedSorted for item in sublist]

	res = {}

	for url_id in byCommunityFlatten :
		res[url_id] = urls[url_id]

	return res			


def toPartitions() :
	
	# create the graph
	graph = nx.Graph()
	graph.add_edges_from(edges)
	
	# find the communities
	communities = community_louvain.best_partition(graph)
	
	return sortUrlsInPartitions(dict(sorted(communities.items(), key=lambda item: item[1])))

partitions = toPartitions()

# join the relevant files and compute the cernes

timestamps = []

# archives

for url_id in partitions :

	url = ids[url_id]
	
	if url in urls_moments :
		status  = urls_moments[url]['status']
		moments = urls_moments[url]['moments']

		for moment in moments :
			timestamps = timestamps + moment[1]

		cernes['urls'][url] = {'id':url_id, 'community':partitions[url_id], 'moments':moments, 'status':status}

cernes['from'] = min(timestamps)[0:4]
cernes['to']   = max(timestamps)[0:4]

# find centroids in cliques

ids_moments = {}
alpha_step  = 360 / len(cernes['urls'])
alpha = 0

for url in cernes['urls'] :
	alpha += alpha_step
	for moment in cernes['urls'][url]['moments'] :
		ids_moments[moment[0]] = {'stamps':moment[1],'alpha':alpha}

centroids = []
memory = []

for clique in cliques['cliques'] :
	if len(clique) > 1 :
		for source in clique :
			for target in clique :
				stamp = []
				alpha = []
				ids = []
				if source == target :
					continue
				if source > target :
					continue
				edge = str(source) + "-" + str(target)
				if source in ids_moments and target in ids_moments and edge in edgesMoments and edge not in memory :
					memory.append(edge)
					stamp.append([ids_moments[source]['stamps'][0],ids_moments[source]['stamps'][1]])
					stamp.append([ids_moments[target]['stamps'][0],ids_moments[target]['stamps'][1]])
					alpha.append(ids_moments[source]['alpha'])
					alpha.append(ids_moments[target]['alpha'])
					ids.append(source)
					ids.append(target)
					centroids.append({'alpha':alpha, 'stamps':stamp, 'moments':ids})

cernes['cliques'] = centroids

# compute the temporal distribution of the urls

timestamps.sort()
distribution = {}

for stamp in timestamps :
	year = int(stamp[0:4]) + 1
	distribution[year] = 0

for url in cernes['urls'] :
	fst_year = int(cernes['urls'][url]['moments'][0][1][0][0:4]) + 1
	distribution[fst_year] = distribution[fst_year] + 1 

cernes['distribution'] = distribution

# export the cernes

with open("output/cernes-" + root + "-" + inf + "-" + sup + ".json", 'w') as file :
   
   json.dump(cernes, file, sort_keys=False, indent=4)