#######
# linksToSiteTree.py
# description : reconstruct the tree like structure of a Web site from its archived and unarchived pages
# licence : AGPL + CECILL v3
# author : quentin lobbé - qlobbe@iscpif.fr
#######

# python3 linksToSiteTree.py output/archives-firsttuesday.com-1996-2010.csv output/unarchived-1996-2010.csv

import sys
import csv
import re
import json

pathArchives = ""
pathUnArchives = ""

try :
    pathArchives = sys.argv[1]
    pathUnArchives = sys.argv[2]
except :
    print ("! args error\n")
    sys.exit(0)

inf = pathArchives.split('-')[2]
sup = pathArchives.split('-')[3].replace(".csv","")

urls = set()
root = ""

def normalizedUrl(url) :
    url = url.replace("http://","").replace("www.","").replace(":80","")
    if url.endswith("/") :
        url = url[:-1]
    if (url.endswith(".htm")) :
        url = url.replace(".htm",".html")
    return url.lower()

# extract the urls of the archived pages

with open(pathArchives, newline='\n') as file :

    rawArchives = csv.reader(file, delimiter=';')
    next(rawArchives)

    for row in rawArchives :

        url = normalizedUrl(row[2])

        urls.add(url)

iterator = iter(urls)
root = str(next(iterator, None)).split('/')[0]

# extract the urls of the unarchived pages

with open(pathUnArchives, newline='\n') as file :

    rawUnArchived = csv.reader(file, delimiter=';')
    next(rawUnArchived)

    for row in rawUnArchived :

        url = root + normalizedUrl(row[2])

        urls.add(url)

# init the siteTree

cpt = 0
siteTree = dict()
siteTree[root] = {'vertice':root, 'depth':0, 'children':{}, 'url':root, 'id':cpt}

# recursively feed the siteTree

def toTree(vertices,tree,depth,url,id) :

    chd = tree['children']

    if not vertices :
        cpt = id
        return 

    fst, rst = vertices[0], vertices[1:]

    if fst not in chd :
        chd[fst] = {'vertice':fst, 'depth':depth, 'children':{}, 'url':url,'id':id}
        id +=1

    
    toTree(rst,chd[fst],depth + 1,url,id)

# split the urls

for url in urls :

    vertices = re.split('[/?&#]',url)
    cpt += 1
    toTree(vertices[1:],siteTree[root],1,url,cpt)

# write the siteTree as a json file

with open("output/siteTree-" + inf + "-" + sup + ".json", 'w') as file :
   
   json.dump(siteTree, file, sort_keys=True, indent=4)