#######
# momentsToGraph.py
# description : find continuity links between archived moments
# licence : AGPL + CECILL v3
# author : quentin lobbé - qlobbe@iscpif.fr
#######

# python3 momentsToGraph.py output/moments-1996-2010.json output/siteTree-1996-2010.json output/linksInternals-1996-2010.csv

import sys
import json
import csv
import re

pathMoments = ""
pathTree    = ""
pathLinks   = ""

try :
    pathMoments = sys.argv[1]
    pathTree    = sys.argv[2]
    pathLinks   = sys.argv[3]
except :
    print ("! args error\n")
    sys.exit(0)

inf = pathMoments.split('-')[1]
sup = pathMoments.split('-')[2].replace(".json","")

def normalizedUrl(url) :
    url = url.replace("http://","").replace("www.","").replace(":80","")
    if url.endswith("/") :
        url = url[:-1]
    if (url.endswith(".htm")) :
        url = url.replace(".htm",".html")
    if url.startswith('/') :
        url = root + url
    if url.startswith('./') :
        url = root + url[1:]  
    if url.startswith('../') :
        url = root + url[2:]  
    return url.lower()

def readJson(path) :
	file = open(path)
	return json.load(file)

def readCsv(path) :
	res = {}
	with open(path, newline='\n') as file :
		rawLinks = csv.reader(file, delimiter=';')
		next(rawLinks)
		for row in rawLinks :
			source = normalizedUrl(row[2])
			target = normalizedUrl(row[4])
			if source not in res :
				res[source] = [target]
			else :
				if target not in res[source] :
					res[source].append(target)
	return res

def filterSiblings(candidates) :
	filtered = {}
	siblings = []
	for vertice in candidates :
		if not candidates[vertice]['children'] :
			filtered[vertice] = candidates[vertice]
		else :
			sbl = candidates[vertice]['children']
			for v in sbl :
				if not sbl[v]['children'] :
					filtered[v] = sbl[v]
	for vertice in filtered :
		siblings.append(filtered[vertice]['url'])
	return siblings

def getSiblings(vertices,tree) :
	if len(vertices) > 0 :
		fst = vertices.pop(0)
		return getSiblings(vertices,tree[fst]['children'])
	else :
		return filterSiblings(tree)

# open the json files

tree  = readJson(pathTree)
root  = next(iter(tree))
pages = readJson(pathMoments)
links = readCsv(pathLinks)
edges = []
edgesByUrls = []
linksByUrls = []

for url in pages :

	vertices = re.split('[/?&#]',url)
	siblings = []
	isThereEdges = False

	# if not the root
	if len(vertices) > 1 :

		# replace "" by the root
		if vertices[0] == "" :
			vertices[0] = root

		# remove the leaf
		lst = vertices.pop()

		# find the siblings
		siblings = getSiblings(vertices,tree)
		if url in siblings :
			siblings.remove(url)

	# find the connected urls
	if url in links :
		siblings = siblings + links[url]
		
	siblings = list(set(siblings))

	# find the continuity edges
	sources = pages[url]['moments']

	# for each source's moments
	for source in sources :
		
		s_id = source[0]
		s_moment = source[1]

		# for each siblings
		for sibling in siblings :

			if sibling in pages :
				targets = pages[sibling]['moments']
				
				# for each targets's moments
				for target in targets :

					t_id = target[0]
					t_moment = target[1]

					# is there any intersection ?
					if s_moment[1] < t_moment[0] :
						continue

					if t_moment[1] < s_moment[0] :
						continue

					if s_id < t_id :
						isThereEdges = True
						edges.append((s_id,t_id))
						edgesByUrls.append((url,sibling))
					else :
						isThereEdges = True
						edges.append((t_id,s_id))
						edgesByUrls.append((sibling,url))

	if not isThereEdges :
		edges.append((pages[url]['moments'][0][0],pages[url]['moments'][0][0]))
		edgesByUrls.append((url,url))

edges = list(set(edges))
edgesByUrls = list(set(edgesByUrls))

# export the continuity edges

csv = open("output/continuity-graph-" + inf + "-" + sup + ".csv", "w")
header = "source;target\n"
csv.write(header)

for edge in edges :
	row = str(edge[0]) + ";" + str(edge[1]) + "\n";
	csv.write(row)

# aggregate the continuity links by urls

csv = open("output/continuity-graph-urls-" + inf + "-" + sup + ".csv", "w")
header = "source;target;source url;target url\n"
csv.write(header)

for source in links :
	for target in links[source] :
		linksByUrls.append((source,target))

linksByUrls = list(set(linksByUrls))
edgesByUrls.extend(linksByUrls)

for edge in edgesByUrls :

	source = edge[0]
	target = edge[1]

	if source in pages and target in pages :

		id_source = pages[source]['id']
		id_target = pages[target]['id']

		row = str(id_source) + ";" + str(id_target) + ";" + source + ";" + target + "\n";
		csv.write(row)